local AchievementTracker = ZO_Object:Subclass()


AchievementTracker.defaults = {
   ["tracked"] = {},
   ["showDetails"] = {},
   ["position"] = {
      ["offsetX"] = 1000,
      ["offsetY"] = 1000,
   },
   ["fontSize"] = {
      ["name"] = 14,
      ["desc"] = 12
   },
   ["size"] = {
		["sizeX"] = 400,
		["sizeY"] = 200
   },
   ["locked"] = false,
   ["showIcons"] = true,
   ["showBackground"] = true,
   ["showDesc"] = true,
   ["showTrackedAchievementCount"] = true,
   ["notify"] = false,
   ["notifyType"] = "alert",
   ["autoTrackZoneAchievements"] = false,
   ["hideOldZoneAchievements"] = true,
   ["hideCompleted"] = true,
   ["bgAlpha"] = 100,
   ["maxTracked"] = 0,
   ["hidden"] = false
}
AchievementTracker.config = {}
AchievementTracker.config.tracked = {}
AchievementTracker.lastZone = nil
AchievementTracker.checkboxesCreated = false
AchievementTracker.heightPerLine = 50
AchievementTracker.bufferTable = {}

local achievementFrame

-- from wykkyd
function AchievementTracker:Buffer(key, buffer)
  if key == nil then return end
  if self.bufferTable[key] == nil then self.bufferTable[key] = {} end
  self.bufferTable[key].buffer = buffer or 3
  self.bufferTable[key].now = GetFrameTimeMilliseconds()
  if self.bufferTable[key].last == nil then self.bufferTable[key].last = self.bufferTable[key].now end
  self.bufferTable[key].diff = self.bufferTable[key].now - self.bufferTable[key].last
  self.bufferTable[key].eval = self.bufferTable[key].diff >= self.bufferTable[key].buffer
  if self.bufferTable[key].eval then self.bufferTable[key].last = self.bufferTable[key].now end
  return self.bufferTable[key].eval
end

function AchievementTracker:GetLabel(name, parent, isBold)
   local label = parent:GetNamedChild(name) or WINDOW_MANAGER:CreateControl(parent:GetName() .. name, parent, CT_LABEL)
   label:SetHidden(false)
   label:SetDimensions(AchievementTracker:getX(), 25) 
   label:SetWrapMode(TEXT_WRAP_MODE_ELLIPSIS)

   local fontFile = nil
   local fontSize = self.config.fontSize["desc"]
   local fontDecoration = "soft-shadow-thin"
   if isBold then
      fontSize = self.config.fontSize["name"]
      fontFile = ZoFontGameBold:GetFontInfo()
   else
      fontFile = ZoFontGame:GetFontInfo()
   end

   label:SetFont(string.format("%s|%d|%s", fontFile, fontSize, fontDecoration))
   return label
   
end

function AchievementTracker:GetIcon(name, parent, texture)
   local ico = parent:GetNamedChild(name) or WINDOW_MANAGER:CreateControl(parent:GetName() .. name, parent, CT_TEXTURE)
   ico:SetHidden(false)
   ico:SetDimensions(15, 15)
   ico:SetTexture(texture)
   return ico
end

function AchievementTracker:AutoTrackZoneAchievements()
   if self.config == nil then return end

   if self.config.autoTrackZoneAchievements then
      local currentZone = GetUnitZone("player")
      if currentZone ~= self.lastZone or self.lastZone == nil then
         if self.lastZone ~= nil and self.config.hideOldZoneAchievements then
            self:FindZoneAchievements(self.lastZone, false)
         end
         self:FindZoneAchievements(currentZone, true)
         self.lastZone = currentZone
      end
   end
end

function AchievementTracker:FindZoneAchievements(zone, track)
   local numCats = GetNumAchievementCategories()

   local function FindAndTrack(cat, subCat, index)
      local id = GetAchievementId(cat, subCat, index)
      local name, desc = GetAchievementInfo(id)
      if string.find(name, zone) ~= nil or string.find(desc, zone) ~= nil then
         if track then
            self.config.tracked[id] = true
            self.config.showDetails[id] = true
         else
            self.config.tracked[id] = nil
            self.config.showDetails[id] = nil
         end
      end
   end

   for iCat = 1, numCats do
      local _, numSubCats, numAchievs = GetAchievementCategoryInfo(iCat)
      if numSubCats > 0 then
         for iSubCat = 1, numSubCats do
            local _, numAchievs = GetAchievementSubCategoryInfo(iCat, iSubCat)
            for iAchiev = 1, numAchievs do
               FindAndTrack(iCat, iSubCat, iAchiev)
            end
         end
      else
         for iAchiev = 1, numAchievs do
            FindAndTrack(iCat, nil, iAchiev)
         end
      end
   end
end

function AchievementTracker:CreateWindow()
   
   -- create top level frame
   self.frame = WINDOW_MANAGER:CreateTopLevelWindow("AchievementTrackerWindow")
   achievementFrame = self.frame
   achievementFrame:SetMouseEnabled(true)
   
   -- if self.config.locked then
		-- achievementFrame:SetMovable(false)
	-- else
		achievementFrame:SetMovable(true)
	-- end
	
	achievementFrame:SetDimensions( AchievementTracker:getX(), AchievementTracker:getY() )
	achievementFrame:SetResizeHandleSize(8)
	achievementFrame:SetAnchor( TOPLEFT, GuiRoot, TOPLEFT, AchievementTracker.getPosX(), AchievementTracker.getPosY() )
		
   
   
	--Set a background to make the window look nice and have a definite shape when moused over.
	self.frame.bg = WINDOW_MANAGER:CreateControl(nil, achievementFrame, CT_BACKDROP)
	achievementFrame.bg:SetHidden( false )
	achievementFrame.bg:SetCenterColor(0,0,0,0.5)
	achievementFrame.bg:SetEdgeColor(.1,.1,.1,1)
	achievementFrame.bg:SetEdgeTexture("",8,1,2)
	achievementFrame.bg:SetAnchorFill(achievementFrame)
	
    achievementFrame:SetHandler( "OnMouseExit" , function() AchievementTracker.MouseExit(achievementFrame) end)
	-- achievementFrame:SetHandler( "OnMouseEnter", function() AchievementTracker.ShowFaded() end) --If mouse goes in window call function to show faded text and background.
	-- achievementFrame:SetHandler("OnMouseDown", function() AchievementTracker.ShowResizeBox() end)
	--achievementFrame:SetHandler("OnMouseUP", function() AchievementTracker.HideResizeBox() end)
	
   achievementFrame:SetHandler("OnMouseUp", function()
      local sizeX = self.frame:GetWidth()
      local sizeY = self.frame:GetHeight()
      -- self.frame:SetDimensions(sizeX, sizeY)
      self.config.position.offsetX = self.frame:GetLeft()
      self.config.position.offsetY = self.frame:GetTop()
	  AchievementTracker:Resize(sizeX, sizeY)
   end)
        
end


function AchievementTracker:Resize(X, Y)

	AchievementTracker.config.size.sizeX = X
	AchievementTracker.config.size.sizeY = Y
	--achievementFrame:SetDimensions(X, Y)
	achievementFrame.bg:SetDimensions(X, Y)
	
	self:LoadTrackedAchievements()
	achievementFrame.bg:SetDimensions(X, Y)
	
end


--Handles what happens when the mouse exits the loot window.
function AchievementTracker.MouseExit(window)

	--AchievementTracker.config.position.xPos = window:GetLeft() -- Set player variables for window position
	--AchievementTracker.config.position.yPos = window:GetTop() -- Set player variables for window position
	
end

function AchievementTracker:getX()
	return AchievementTracker.config.size.sizeX 
end

function AchievementTracker:getY()
	return AchievementTracker.config.size.sizeY 
end

function AchievementTracker:getPosX()
	return AchievementTracker.config.position.offsetX
end
function AchievementTracker:getPosY()
	return AchievementTracker.config.position.offsetY
end


function AchievementTracker:LoadTrackedAchievements(updatedId)
   -- check options and act accordingly
   
   if self.config.showBackground then
      achievementFrame.bg:SetHidden(false)
      achievementFrame.bg:SetAlpha(self.config.bgAlpha / 100)
   else
      achievementFrame.bg:SetHidden(true)
   end
   if self.config.autoTrackZoneAchievements then
      self:AutoTrackZoneAchievements(GetUnitZone("player"))
   end
	
	if not (self.config.tracked[0] == nil) then
	   if self.config.hidden then
		  self.sceneFragment:Hide()
	   else
		  self.sceneFragment:Show()
	   end
   end

   -- notify
   if updatedId ~= nil and updatedId > 0 and self.config.notify then
      local name, desc, _, icon, done = GetAchievementInfo(updatedId)
      if name ~= "" and name ~= nil then
         local numCriteria = GetAchievementNumCriteria(updatedId)
         local totalCompleted = 0
         local totalRequired = 0
         for criteria = 1, numCriteria do
            local _, critCompleted, critRequired = GetAchievementCriterion(updatedId, criteria)
            totalCompleted = totalCompleted + critCompleted
            totalRequired = totalRequired + critRequired
         end

         local quarterNeeded = totalRequired * 0.25
         local halfNeeded = totalRequired * 0.5
         local threeQuartersNeeded = totalRequired * 0.75

         local showUpdate = false
         if (totalCompleted == quarterNeeded or totalCompleted == halfNeeded or totalCompleted == threeQuartersNeeded) or totalRequired <= 100 then
            showUpdate = true
         end

         local finished = false
         if totalRequired == totalCompleted then
            showUpdate = true
            finished = true
         end

         totalCompleted = ZO_CommaDelimitNumber(totalCompleted)
         totalRequired = ZO_CommaDelimitNumber(totalRequired)

         if showUpdate then
            if finished then
               if self.config.notifyType == "alert" then
                  ZO_Alert(UI_ALERT_CATEGORY_ALERT, SOUNDS.DEFAULT_CLICK, zo_strformat("Achievement |cFF4136<<1>>|r completed.", name))
               elseif self.config.notifyType == "chat" then
                  d(zo_strformat("Achievement |cFF4136<<1>>|r completed.", name))
               end
            else
               if self.config.notifyType == "alert" then
                  ZO_Alert(UI_ALERT_CATEGORY_ALERT, SOUNDS.DEFAULT_CLICK, zo_strformat("Achievement |cFF4136<<1>>|r advanced. (<<2>>/<<3>>)", name, totalCompleted, totalRequired))
               elseif self.config.notifyType == "chat" then
                  d(zo_strformat("Achievement |cFF4136<<1>>|r advanced. (<<2>>/<<3>>)", name, totalCompleted, totalRequired))
               end
            end
         end
      end
   end


   -- no need to do all the stuff below if frame is hidden
   if self.sceneFragment:IsHidden() then return end

   local i = 1
   local lastTotalHeight = 0;

   -- traverse tracked achievs and display them
   for id, isTracked in pairs(self.config.tracked) do
      local name, desc, _, icon, done = GetAchievementInfo(id)

      -- check if done
      local continue = true
      if done and self.config.hideCompleted then
         continue = false
         self.config.tracked[id] = nil
         self.config.showDetails[id] = nil
      end

      local max = self.config.maxTracked
      if self.config.maxTracked == 0 then max = 9999 end

      if i > max then return end

		  if isTracked and continue then
			 local offsetY = 5
			 if lastTotalHeight > 0 then offsetY = lastTotalHeight end
			 
			 -- icon
			 local achievIcon = self:GetIcon("Icon" .. i .. "\n", achievementFrame, icon)
			 
			 achievIcon:SetAnchor(TOPLEFT, achievementFrame, TOPLEFT, 5, offsetY)
			 achievIcon:SetHidden(not self.config.showIcons)
			 
			 
			 -- name
			 local achievName = self:GetLabel("Label" .. i, achievementFrame, true)
			 local xPos = 5
			 --if achievIcon:IsHidden() then xPos = -15 end
			 achievName:SetHeight(15)
			 achievName:SetAnchor(TOPLEFT, achievIcon, TOPRIGHT, xPos, 0)
			 achievName:SetText(name)
			 achievName:SetMouseEnabled(true)
			 if done then
				achievName:SetColor(0, 1, 0, 1)
			 else
				achievName:SetColor(1, 1, 1, 1)
			 end

			 achievName:SetHandler("OnMouseEnter", function(self)
				achievName:SetColor(1, 0.86, 0, 1)
			 end)
			 achievName:SetHandler("OnMouseExit", function(self)
				if done then
				   achievName:SetColor(0, 1, 0, 1)
				else
				   achievName:SetColor(1, 1, 1, 1)
				end
			 end)
			 achievName:SetHandler("OnMouseUp", function(_, button)
				if button == 1 then
				   ACHIEVEMENTS:ShowAchievement(id) --it has to be called before scene is shown
				   SCENE_MANAGER:Show("achievements")
				elseif button == 3 then
				   
				   self.config.tracked[id] = nil
				   self.config.showDetails[id] = nil
				
				   return self:LoadTrackedAchievements()
				elseif button == 2 then
				   -- achievName.showDetails = not achievName.showDetails
				   self.config.showDetails[id] = not self.config.showDetails[id]
				   return self:LoadTrackedAchievements()
				end
			 end)
			 
			 -- description
			 local achievDesc = self:GetLabel("Desc" .. i, achievementFrame)
			 achievDesc:SetHeight(100)
			 
			 achievDesc:SetWidth(achievementFrame:GetWidth() - (achievementFrame:GetWidth()/5))
			 achievDesc:SetAnchor(TOPLEFT, achievName, TOPLEFT, 10, achievName:GetTextHeight())
			 if done then
				achievDesc:SetColor(0, 1, 0, 1)
			 else
				achievDesc:SetColor(0.8, 0.8, 0.8, 1)
			 end

			 if self.config.showDesc then
				achievDesc:SetText(desc)
			 else
				achievDesc:SetText("")
			 end

			 -- criteria
			 local achievCriteria = self:GetLabel("Criteria" .. i, achievementFrame)
			 achievCriteria:SetAnchor(TOPLEFT, achievDesc, TOPLEFT, 0, achievDesc:GetTextHeight())
			 if done then
				achievCriteria:SetColor(0, 1, 0, 1)
			 else
				achievCriteria:SetColor(0.8, 0.8, 0.8, 1)
			 end

			 local numCriteria = GetAchievementNumCriteria(id)
			 local totalCompleted = 0
			 local totalRequired = 0

			 -- Added by Maikahn
			 -- if the option to show the detail has been selected, process the criterion data for display
			 --if self.config.showDetails[id] then
				local text = ""
				local critDisplayed = 0
				for criteria = 1, numCriteria do
				   local critDesc, critCompleted, critRequired = GetAchievementCriterion(id, criteria)

				   if (critCompleted ~= critRequired) or (critCompleted == critRequired and not self.config.hideCompleted) then
					  -- If this is the second or higher, then add a line break prior to the data
					  critDisplayed = critDisplayed + 1
					  if critDisplayed > 1 then
						 text = text .. "\n"
					  end

					  if critCompleted ~= critRequired then
					  -- Don't bother showing 0/1 if only one item is required and it is not complete.
						 if critRequired == 1 then
							text = text .. " - " .. critDesc
						 else
							text = text .. " - " .. critDesc .. " : " .. critCompleted .. "/" .. critRequired
						 end
					  -- if the criterion is complete, but the settings are set to display, show it in the list
					  else
						 text = text .. "|c2ECC40 - " .. critDesc .. "|r"
					  end				  
				   end			   
				end
				text = text .. "\n\n"
				achievCriteria:SetText(text)
				achievCriteria:SetWidth(achievementFrame:GetWidth() - (achievementFrame:GetWidth()/5))
				achievCriteria:SetHeight((achievCriteria:GetTextHeight() * numCriteria)+15)

				-- If we are just showing the overall total, then get the total of all criterion.
			 -- else
				-- for criteria = 1, numCriteria do
				   -- local critDesc, critCompleted, critRequired = GetAchievementCriterion(id, criteria)
				   -- totalCompleted = totalCompleted + critCompleted
				   -- totalRequired = totalRequired + critRequired
				-- end
				-- if totalCompleted > 1 then
				   -- totalCompleted = ZO_CommaDelimitNumber(totalCompleted)
				-- end
				-- if totalRequired > 1 then
				   -- totalRequired = ZO_CommaDelimitNumber(totalRequired)
				   -- achievCriteria:SetText(" - " .. totalCompleted .. "/" .. totalRequired)
				-- else
				   -- achievCriteria:SetText("")
				-- end
			 -- end
	-- if end
			 -- add the total height of this achiev so the next one gets positioned correctly. plus a 10px padding
			 lastTotalHeight = lastTotalHeight + achievName:GetTextHeight() + achievDesc:GetTextHeight() + achievCriteria:GetTextHeight() + 10

			 i = i + 1
		  end
	  achievementFrame:SetHeight( lastTotalHeight)
   end
   
  
end

function AchievementTracker:AddTrackedAchievementCount()
   if not AchievementTracker:Buffer("AddTrackedAchievementCount", 10) then return end

   local parent = ZO_AchievementsContentsCategoriesScrollChildContainer1
   local numChildren = parent:GetNumChildren()
   -- skip first child because that's the "Summary" tab
   for i = 2, numChildren do
      local child = parent:GetChild(i)
      local txtChild = child:GetNamedChild("Text") or nil

      local function AppendTrackedCount(control, trackCount)
         control:SetFont(string.format("%s|%d|%s", ZoFontGameBold:GetFontInfo(), 21, "soft-shadow-thin"))
         control:SetText(tostring("|cAAAAAA(".. trackCount .. ")|r " .. control.orgText))
      end

      if txtChild ~= nil then
         if txtChild.orgText == nil then
            txtChild.orgText = txtChild:GetText()
         end

         local tracked = 0

         -- tracked: 483 / 769
         -- wenn nil dann erste subcat
         -- count startet bei 2. subcat
         -- GetCategoryInfoFromAchievementId(aId)
         -- Returns: luaindex topLevelIndex, luaindex:nilable categoryIndex, luaindex achievementIndex
         for id, isTracked in pairs(self.config.tracked) do
            if isTracked then
               local catIndex, subCatIndex, achievementIndex = GetCategoryInfoFromAchievementId(id)
               if child.node.data.categoryIndex == catIndex then
                  tracked = tracked + 1
               end
            end
         end

         AppendTrackedCount(txtChild, tracked)
      else
         local function FindTrackedForSubcat(subcatControl, catIndex, subcatIndex)
            subcatControl.tracked = 0
            if subcatControl.orgText == nil then subcatControl.orgText = subcatControl:GetText() end

            for id, isTracked in pairs(self.config.tracked) do
               if isTracked then
                  local _catIndex, _subcatIndex, _ = GetCategoryInfoFromAchievementId(id)
                  if catIndex == _catIndex and subcatIndex == _subcatIndex then
                     subcatControl.tracked = subcatControl.tracked + 1
                  end
               end
            end
         end

         local numSubcats = child:GetNumChildren()
         for j = 1, numSubcats do
            local subcat = child:GetChild(j)
            if j == 1 then
               FindTrackedForSubcat(subcat, subcat.node.data.parentData.categoryIndex, nil)
            else
               FindTrackedForSubcat(subcat, subcat.node.data.parentData.categoryIndex, j - 1)
            end

            AppendTrackedCount(subcat, subcat.tracked)
         end
      end
   end
end

function AchievementTracker:CreateCheckboxes()
   local list = ZO_AchievementsContentsAchievementListScrollChild
   local numOfAchievements = list:GetNumChildren()

   for i = 1, numOfAchievements do
      local achievement = list:GetChild(i)
      if achievement ~= nil then
         local title = achievement:GetNamedChild("Title")
         if title ~= nil then
            title:SetAnchor(3, achievement, 3, 110, 10)

            local id = achievement["achievement"]["achievementId"]

            local controlName = tostring(title:GetName() .. "TrackerCheckbox")

            local cb = title:GetNamedChild("TrackerCheckbox") or WINDOW_MANAGER:CreateControlFromVirtual(controlName, title, "ZO_CheckButton")
            cb:SetAnchor(LEFT, title, LEFT, -20, 0)
            cb:SetDimensions(15, 15)
            cb:SetHidden(false)
            cb:SetHandler("OnMouseUp", function() self:ToggleTracked() end)

            if self.config.tracked[id] == true then
               state = BSTATE_PRESSED
            else
               state = BSTATE_NORMAL
            end

            cb:SetState(state, false)
         end
      end
   end
end

function AchievementTracker:ToggleNotifications()
   local text = ""
   self.config.notify = not self.config.notify
   if self.config.notify then text = "ON" else text = "OFF" end
   ZO_Alert(UI_ALERT_CATEGORY_ALERT, SOUNDS.DEFAULT_CLICK, "Achievement notifications are now " .. text)
end

function AchievementTracker:ToggleWindow()
	achievementFrame:SetHidden(self.config.hidden) 
	self.config.hidden = not self.config.hidden
    AchievementTracker:LoadTrackedAchievements()
end

function AchievementTracker:ToggleTracked()
   local control = WINDOW_MANAGER:GetMouseOverControl()
   local firer = control:GetParent():GetParent()
   local id = firer["achievement"]["achievementId"]

   if self.config.tracked[id] then
      self.config.tracked[id] = nil
      self.config.showDetails[id] = nil
   else
      self.config.tracked[id] = true
      -- self.config.showDetails[id] = false
   end

   self:LoadTrackedAchievements()
end

function AchievementTracker:CheckAchievementJournalVisibility()
   if not AchievementTracker:Buffer("CheckAchievementJournalVisibility", 50) then return end

   if ZO_AchievementsContentsAchievementListScrollChild:IsHidden() == false then
      self:CreateCheckboxes()
   end
end

function AchievementTracker:Reset()
   for id, isTracked in pairs(self.config.tracked) do
      self.config.tracked[id] = nil
      self.config.showDetails[id] = nil
   end

   self:LoadTrackedAchievements()
end

function AchievementTracker:CreateSettingsWindow()
   local panelData = {
      type = "panel",
      name = "Achievement Tracker",
      displayName = ZO_HIGHLIGHT_TEXT:Colorize("Achievement Tracker"),
      author = "Biki",
      version = "1.9.0",
      registerForRefresh = true,
      registerForDefaults = true,
   }

   local optionsData = {
      {
         type = "checkbox",
         name = 'Lock in place',
         tooltip = 'Makes the window unmovable and fixed to its position',
         getFunc = function() return self.config.locked end,
         setFunc = function(state) self.config.locked = state; self:LoadTrackedAchievements() end,
         default = self.defaults.locked,
      },
      {
         type = "checkbox",
         name = 'Show icons',
         tooltip = 'Shows the corresponding icon to the left of the achievement name',
         getFunc = function() return self.config.showIcons end,
         setFunc = function(state) self.config.showIcons = state; self:LoadTrackedAchievements() end,
         default = self.defaults.showIcons,
      },
      {
         type = "checkbox",
         name = 'Show description',
         tooltip = 'Shows the description text underneath the name. Toggle to save space',
         getFunc = function() return self.config.showDesc end,
         setFunc = function(state) self.config.showDesc = state; self:LoadTrackedAchievements() end,
         default = self.defaults.showDesc,
      },
      {
         type = "checkbox",
         name = 'Notify progress',
         tooltip = 'Notifies you when you advance any achievement',
         getFunc = function() return self.config.notify end,
         setFunc = function(state) self.config.notify = state end,
         default = self.defaults.notify,
      },
      {
         type = "dropdown",
         name = ' - Type',
         tooltip = 'Select the type of notification you wish to receive',
         choices = {"alert", "chat"},
         getFunc = function() return self.config.notifyType end,
         setFunc = function(value) self.config.notifyType = value end,
         default = self.defaults.notifyType,
		disabled = function() return not self.config.notify end, 
         reference = "notifyType",
      },
      {
         type = "checkbox",
         name = 'Auto track zone achievements',
         tooltip = 'Automatically tries to detect achievements of the current zone you are in and tracks them',
         getFunc = function() return self.config.autoTrackZoneAchievements end,
         setFunc = function(state) self.config.autoTrackZoneAchievements = state; self.lastZone = nil; self:LoadTrackedAchievements() end,
         default = self.defaults.autoTrackZoneAchievements,
      },
      {
         type = "checkbox",
         name = ' - Hide old zone achievements',
         tooltip = 'Automatically hides achievements from the old zone when you change zones',
         getFunc = function() return self.config.hideOldZoneAchievements end,
         setFunc = function(state) self.config.hideOldZoneAchievements = state; self.lastZone = nil; self:LoadTrackedAchievements() end,
		disabled = function() return not self.config.autoTrackZoneAchievements end, 
         default = self.defaults.hideOldZoneAchievements,
      },
      {
         type = "checkbox",
         name = 'Show tracked achievement count',
         tooltip = 'Shows the total tracked achievement count inside the achievement journal so you can easier see in which categories you are tracking something',
         getFunc = function() return self.config.showTrackedAchievementCount end,
         setFunc = function(state) self.config.showTrackedAchievementCount = state; ReloadUI("ingame") end,
         warning = "Changing this setting will reload the UI",
         default = self.defaults.showTrackedAchievementCount,
      },
      {
         type = "checkbox",
         name = 'Hide completed',
         tooltip = 'Automatically hides achievements after you complete them. Also disables tracking of already completed Achievs of course',
         getFunc = function() return self.config.hideCompleted end,
         setFunc = function(state) self.config.hideCompleted = state; self.lastZone = nil; self:LoadTrackedAchievements() end,
         default = self.defaults.hideCompleted,
      },
      {
         type = "checkbox",
         name = 'Show background',
         tooltip = 'Shows a background behind the tracker window',
         getFunc = function() return self.config.showBackground end,
         setFunc = function(state) self.config.showBackground = state; self:LoadTrackedAchievements() end,
         default = self.defaults.showBackground,
      },
      {
         type = "slider",
         name = 'Maximum tracked',
         tooltip = 'Adjust the maximum number of tracked achievements. 0 means unlimited',
         min = 0,
         max = 10,
         getFunc = function() return self.config.maxTracked end,
         setFunc = function(value) self.config.maxTracked = value; self:LoadTrackedAchievements() end,
         default = self.defaults.maxTracked,
      },
      {
         type = "slider",
         name = 'Background alpha',
         tooltip = 'Adjust the alpha value of the background',
         min = 0,
         max = 100,
         getFunc = function() return self.config.bgAlpha end,
         setFunc = function(value) self.config.bgAlpha = value; self:LoadTrackedAchievements() end,
         default = self.defaults.bgAlpha,
      },
      {
         type = "slider",
         name = 'Name font size',
         tooltip = 'Adjust the font size of the tracked achievement names',
         min = 8,
         max = 30,
         getFunc = function() return self.config.fontSize["name"] end,
         setFunc = function(value) self.config.fontSize["name"] = value; self:LoadTrackedAchievements() end,
         default = self.defaults.fontSize["name"],
         reference = "fontSizeName",
      },
      {
         type = "slider",
         name = 'Description font size',
         tooltip = 'Adjust the font size of the tracked achievement descriptions',
         min = 8,
         max = 30,
         getFunc = function() return self.config.fontSize["desc"] end,
         setFunc = function(value) self.config.fontSize["desc"] = value; self:LoadTrackedAchievements() end,
         width = "full",
         default = self.defaults.fontSize["desc"],
      },
      {
         type = "button",
         name = 'Untrack all',
         tooltip = 'Untracks all tracked achievements',
         func = function() return self:Reset() end,
      },
   }
   local LAM2 = LibStub("LibAddonMenu-2.0")
   LAM2:RegisterAddonPanel("_bikisAchievementTracker", panelData)
   LAM2:RegisterOptionControls("_bikisAchievementTracker", optionsData)
end

function AchievementTracker:Init(event, name)
   if name ~= "AchievementTracker" then return end

   self.config = ZO_SavedVars:New("AchievementTrackerSavedVars", 3.0, nil, self.defaults)

   -- keybind strings
   ZO_CreateStringId("SI_BINDING_NAME_TOGGLE_ACHIEVEMENT_TRACKER", "Toggle Achievement Tracker")
   ZO_CreateStringId("SI_BINDING_NAME_TOGGLE_ACHIEVEMENT_NOTIFICATIONS", "Toggle Achievement Notifications")

   ZO_Achievements:SetHandler("OnUpdate", function()
      self:CheckAchievementJournalVisibility()
      if self.config.showTrackedAchievementCount then
         self:AddTrackedAchievementCount()
      end
   end)

   self:CreateSettingsWindow()
   self:CreateWindow()

   self.sceneFragment = ZO_FadeSceneFragment:New(achievementFrame)
   self.sceneFragment:RegisterCallback("StateChange", function(oldState, newState)
      if newState == SCENE_FRAGMENT_SHOWING then
         self:LoadTrackedAchievements()
      end
   end)
   HUD_SCENE:AddFragment(self.sceneFragment)
   HUD_UI_SCENE:AddFragment(self.sceneFragment)
   

   -- event hooks
   AchievementTrackerWindow:RegisterForEvent(EVENT_ZONE_CHANGED, function() self:AutoTrackZoneAchievements() end)
   AchievementTrackerWindow:RegisterForEvent(EVENT_ACHIEVEMENT_UPDATED, function(_, id) self:LoadTrackedAchievements(id) end)
	
   -- called after a slight delay. an instant call causes weird spacing errors inside the frame
   zo_callLater(function() self:LoadTrackedAchievements() end, 300)
end

-- called from the keybind
function AchievementTracker_ToggleWindow()
   AchievementTracker:ToggleWindow()
end

function AchievementTracker_ToggleNotifications()
   AchievementTracker:ToggleNotifications()
end

-- public debug functions. can be called via /script AT_update()
function AT_update()
   AchievementTracker:LoadTrackedAchievements()
end

function AT_reset()
   AchievementTracker:Reset()
end

EVENT_MANAGER:RegisterForEvent("AchievementTracker", EVENT_ADD_ON_LOADED, function(...) AchievementTracker:Init(...) end)